@@ -27,11 +27,13 @@ _pygi_import (void)
 
     module = PyImport_ImportModule("gi");
     if (module == NULL) {
+        PyErr_Clear();
         return -1;
     }
 
     api = PyObject_GetAttrString(module, "_API");
     if (api == NULL) {
+        PyErr_Clear();
         Py_DECREF(module);
         return -1;
     }
@@ -49,7 +51,6 @@ _pygi_import (void)
 
     return 0;
 #else
-    PyErr_SetString(PyExc_ImportError, "PyGI support not enabled");
     return -1;
 #endif /* ENABLE_PYGI */
 }
